/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2005 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#include <gtkmm.h>

#include "libtkcommon/exceptions.h"
#include "libtkdb/database.h"
#include "libtkresolver/sdl.h"
#include "catcher.h"

using namespace TkResolver;
using namespace Experimenter;


struct ExceptionDetails
{
  bool          occurred;
  Glib::ustring type;
  Glib::ustring message;

  ExceptionDetails(const Glib::ustring & typ, const Glib::ustring & msg)
    : occurred(true), type(typ), message(msg) {}

  ExceptionDetails()
    : occurred(false) {}
};

bool
TkResolver::Experimenter:: catch_exception(Generator &gen, bool interactive)
{
  ExceptionDetails exception_details;

  try {
    gen();
  }
  catch (const Db::Error &e)
    {
      exception_details = ExceptionDetails("Database error", e.what());
    }
  catch (const UnixError &e)
    {
      exception_details = ExceptionDetails("System error", e.what());
    }
  catch (const std::domain_error &e)
    {
      exception_details = ExceptionDetails("Domain error", e.what());
    }
  catch (const std::invalid_argument &e)
    {
      exception_details = ExceptionDetails("Invalid argument", e.what());
    }
  catch (const std::underflow_error &e)
    {
      exception_details = ExceptionDetails("Underflow error", e.what());
    }
  catch (const std::runtime_error &e)
    {
      exception_details = ExceptionDetails("Runtime error", e.what());
    }
  catch (const std::exception & e)
    {
      exception_details = ExceptionDetails("Error", e.what());
    }
  catch (...)
    {
      exception_details = ExceptionDetails("Unknown error", "Unknown error");
    }
  
  
  SDL::reset();
  
  if (interactive && exception_details.occurred)
    {
      Gtk::MessageDialog dialog(exception_details.message, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
      dialog.set_title(exception_details.type);
      dialog.run();
    }

  return exception_details.occurred;
}
