/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#include <boost/assign/list_of.hpp>

#include "libtkcommon/mbs.h"
#include "location_selector.h"

using namespace TkResolver;
using namespace Experimenter;
using namespace Db::Ui;

const static NameList selection_columns = boost::assign::list_of(Name("name"));

LocationSelector:: LocationSelector(Db::Database * database,
				    Gtk::Window * parent)
  : DataSelectorInputList<LocationInput> (database,
					  "location",
					  selection_columns,
					  parent,
					  "Location")
{
  const static ListColumnList display_columns = boost::assign::list_of(ListColumnInfo(Name("Name"), selection_columns, 0));
  
  set_display_columns(display_columns);
  set_selection("name", database->choose_location());
}

TkString
LocationSelector:: get_name() const
{
  /*  Selection selections(get_selection());
  DEBUG_OUT << "location selection size: " << selections.size() << std::endl;
  const Db::Values selection(get_selection().front());
  std::string name(selection.find("name")->second);
  return name;*/

  return std::string(get_selection().front()["name"]);
}



LocationInput:: LocationInput(Gtk::Window & parent)
  : DataInput("Location details", parent) 
{
  Gtk::Table * table = Gtk::manage(new Gtk::Table(3, 2));
  get_vbox()->pack_start(*table, true, true);
  
  table->attach(*Gtk::manage(new Gtk::Label("Name",    Gtk::ALIGN_LEFT)), 0, 1, 0, 1, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Address", Gtk::ALIGN_LEFT)), 0, 1, 1, 2, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Height",  Gtk::ALIGN_LEFT)), 0, 1, 2, 3, Gtk::FILL, Gtk::FILL);
  
  _height.set_numeric(true);
  Gtk::Adjustment * adj = Gtk::manage(new Gtk::Adjustment(0, -10000, 10000));
  _height.set_adjustment(*adj);

  table->attach(_name,    1, 2, 0, 1, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_address, 1, 2, 1, 2, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_height,  1, 2, 2, 3, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);

  show_all_children();
}


TkResolver::Db::Parameters *
LocationInput:: get_data_sub()
{
  Db::Parameters * parameters = new Db::Parameters;

  parameters->insert(Db::Parameter("name",    Db::String(_name.get_text())));
  parameters->insert(Db::Parameter("address", Db::String(_address.get_text())));
  parameters->insert(Db::Parameter("height",  Db::String(_height.get_value_as_int())));

  return parameters;
}
