/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_VALUE_H__
#define __TK_VALUE_H__

#include <stdexcept>

#include "tkstring.h"

namespace TkResolver {
  
  template <typename T>
    class TkValue
    {
    private:
      T    *_value;
      
    public:
      bool is_null() const
	{ return _value == 0; }
      bool is_valid() const
	{ return !is_null(); }
      
    private:
      void check_null() const
	{ if (is_null()) throw std::runtime_error("value is null"); }
      
    public:
      TkValue()
	: _value(0) {}
      TkValue(const T &t)
	: _value(new T(t)) {}
      TkValue(const T *t)
	: _value(t ? new T(*t) : 0) {}
      TkValue(const TkValue &v)
	: _value( v.is_null() ? 0 : new T(v.get_value()) ) {}
      
      const T *get_value_ptr() const
	{ return _value; }
      T *get_value_ptr()
	{ return _value; }
      
      T &get_value()
	{ check_null(); return *_value;}
      const T &get_value() const
	{ check_null(); return *_value;}
      
      void clear()
	{ if (is_valid()) { delete _value; _value = 0; } }
      
      TkValue &operator= (const TkValue &v)
	{ clear(); _value = v.is_null() ? 0 : new T(v.get_value()); return *this; }
      T &operator= (const T &t)
	{ clear(); _value = new T(t); return *_value; }
      T *operator= (const T *t)
	{ clear(); _value = t ? new T(*t) : 0; return _value; }
      template <typename X>
	TkValue &operator= (const TkValue<X> &v)
	{ if (v.is_null()) clear(); else *this = static_cast<T>(v.get_value()); return *this; }
      
      operator T & ()
	{ return get_value(); }
      operator const T & () const
	{ return get_value(); }
      operator T * ()
	{ return get_value_ptr(); }
      operator const T * () const
	{ return get_value_ptr(); }
      operator bool () const
	{ return is_valid(); }
    };
  
  typedef TkValue<int> TkInt;
  typedef TkValue<unsigned> TkUnsigned;
  typedef TkValue<long> TkLong;
  typedef TkValue<unsigned long> TkUnsignedLong;
  typedef TkValue<float> TkFloat;
  typedef TkValue<double> TkDouble;
  typedef TkValue<bool> TkBool;
  typedef TkValue<TkString> TkStringValue;
  
}; /* namespace TkValue */


#endif /* __TK_NULL_H__ */
