/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_DB_UI_DATA_SELECTOR_H__
#define __TK_DB_UI_DATA_SELECTOR_H__

#include <list>
#include <stdexcept>

#include <gtkmm.h>

#include "libtkcommon/debug.h"
#include "libtkdb/database.h"

namespace TkResolver {
  namespace Db {
    namespace Ui {

      class NoSelection : std::runtime_error
	{
	public:
	  NoSelection();
	};

      typedef Glib::ustring Name;
      typedef std::list<Name> NameList;

      NameList name_list_merge(const NameList &, const NameList &);
      NameList name_list_uniq (const NameList &);

      typedef std::list<Gtk::TreeModel::const_iterator> ModelIteratorList;
      typedef std::list<Db::Values> Selection;

      typedef std::pair<Name, Gtk::TreeModelColumnBase *> TreeModelColumnInfo;
      typedef std::list<TreeModelColumnInfo> TreeModelColumnList;
      
      template<typename S>
      class DisplayColumnInfo
	{
	public:
	  Name     title;
	  NameList columns;
	  S *      slot;
	  
	  DisplayColumnInfo(const Name & t, const NameList & cs, S * s)
	    : title(t), columns(cs), slot(s) {}
        };
      
      template<typename C>
      class DisplayColumnAdaptor
	{
	private:
	  C * _column;
	public:
	  DisplayColumnAdaptor(C * column);
	  void pack_start(Gtk::CellRenderer & cell);
	  void set_cell_data_func(Gtk::CellRenderer & cell, typename C::SlotCellData & slot);
	  void add_attribute(Gtk::CellRenderer & cell, const Glib::ustring& attribute, int column_index);
	};
      
      template<typename S>
	NameList display_column_names(const std::list< DisplayColumnInfo<S> > & display_columns);
      
      class DataSelector
	{
	private:
	  Db::Database *                        _database;
	  Name                                  _table_name;
	  NameList                              _selection_columns;
	  TreeModelColumnList                   _store_columns;
	  Glib::RefPtr<Gtk::ListStore>          _store;
	  
	  Name get_column_type_name(const Name & column_name) const;
	  
	  
	  Gtk::TreeModelColumnBase * create_model_column(const Name & name) const;
	  Name get_column_view_attribute(const Name & name) const;
	  Gtk::CellRenderer * create_cell(const Name & name) const;
	  
	  TreeModelColumnList::const_iterator get_store_column(const Name & name) const;
	  Gtk::TreeModelColumnBase * get_model_column(const Name & name) const;

	  
	  void append_row(const Db::Row & row);
	  Db::NameList store_db_columns();
	  
	protected:
	  DataSelector(Db::Database * database,
		       const Name & table_name,
		       const NameList & selection_columns);
	  
	  // should be called immediately after constructor
	  void set_columns(const NameList & display_columns);

	  Db::Database * get_database() const;
	  const Name & get_table_name() const;
	  const NameList & get_selection_columns() const;
	  const Glib::RefPtr<Gtk::ListStore> & get_store() const;
	  Db::Value get_column_value(Gtk::TreeModel::iterator & iter, const Glib::ustring &column_name) const;
	  
	  unsigned get_column_index(const Name & column_name) const;
	  
	  template<typename C>
	    Gtk::CellRenderer * create_view_cell(const DisplayColumnInfo<typename C::SlotCellData> &, DisplayColumnAdaptor<C> &);

	  virtual ModelIteratorList get_selected() const = 0;
	  Selection get_selection() const;
	  
	  void refresh();
	  virtual void refreshed() = 0;
	  virtual void select(Gtk::TreeModel::iterator &iter) = 0;
	public:
	  void set_selection(const TkString &column_name, const TkString &value);
	};


        #include "data_selector_impl.h"

      
    } /* namespace Ui */
  } /* namespace Db */
} /* namespace TkResolver */




#endif /* __TK_DB_UI_DATA_SELECTOR_H__ */
