/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#include "login.h"

using namespace TkResolver;
using namespace Db::Ui;


static const TkString login_conf_base("/apps/tkresolver/login/");

static TkString
get_conf_value(const Glib::RefPtr<Gnome::Conf::Client> &conf,
	       const TkString &value,
	       const TkString &default_value)
{
  Gnome::Conf::Value conf_value;
  TkString key(login_conf_base + value);

  try
    {
      conf_value = conf->get(key);
    }
  catch (...)
    {
      return default_value;
    }
  
  if (conf_value.get_string().length() < 1)
    return default_value;
  
  return conf_value.get_string();
}

Login:: Login()
  : Gtk::Dialog("Database login"),
    _conf(Gnome::Conf::Client::get_default_client())
{
  get_vbox()->set_spacing(4);
  set_has_separator();
  Gtk::Button * ok_button( add_button(Gtk::Stock::OK,     Gtk::RESPONSE_OK) );
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  set_default_response(Gtk::RESPONSE_OK);

  _hostname.set_activates_default();
  _database.set_activates_default();
  _username.set_activates_default();
  _password.set_activates_default();
  _hostname.set_text(get_conf_value(_conf, "hostname", "localhost"));
  _database.set_text(get_conf_value(_conf, "database", "psi"));
  _username.set_text(get_conf_value(_conf, "username", "psi"));
  _password.set_visibility(false);
  
  Gtk::Table * table = Gtk::manage(new Gtk::Table(4, 2));
  get_vbox()->pack_start(*table, true, true);
  table->set_spacings(4);
  
  table->attach(*Gtk::manage(new Gtk::Label("Hostname",    Gtk::ALIGN_LEFT)), 0, 1, 0, 1, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Database",    Gtk::ALIGN_LEFT)), 0, 1, 1, 2, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Username",    Gtk::ALIGN_LEFT)), 0, 1, 2, 3, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Password",    Gtk::ALIGN_LEFT)), 0, 1, 3, 4, Gtk::FILL, Gtk::FILL);
  
  table->attach(_hostname, 1, 2, 0, 1, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_database, 1, 2, 1, 2, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_username, 1, 2, 2, 3, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_password, 1, 2, 3, 4, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  
  _password.grab_focus();
  show_all_children();
}

Login:: ~Login()
{
  _conf->set(TkString(login_conf_base + L"hostname"), get_hostname());
  _conf->set(TkString(login_conf_base + L"database"), get_database());
  _conf->set(TkString(login_conf_base + L"username"), get_username());
}

#define get_text(func, entry) \
TkString \
Login:: func() const \
{ \
  return entry.get_text(); \
}

get_text(get_hostname, _hostname);
get_text(get_database, _database);
get_text(get_username, _username);
get_text(get_password, _password);
