/* -*- c++ -*-
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so by
 * e-mail - mail your message to Viktor T. Toth at <vttoth@vttoth.com>.
 *
 * Updated 2007 by Robert Ham for tkresolver
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh> for tkresolver
 * Copyright (C) Viktor T. Toth at <vttoth@vttoth.com>
 *
 */

#ifndef __TK_MP_COMPLEX_H__
#define __TK_MP_COMPLEX_H__

#include <complex>

#include <gmpxx.h>

#include "real.h"
#include "matrix.h"

template <typename T, typename X> std::complex<T> operator+(const std::complex<T> &op1, const X &op2);
template <typename T, typename X> std::complex<T> operator/(const X &op1, const std::complex<T> &op2);
template <typename T, typename X> std::complex<T> operator*(const X &op1, const std::complex<T> &op2);

#include "std_complex_impl.h"

namespace TkResolver {
  namespace MP {
    namespace Complex {
	
      typedef std::complex<Real::Number> Number;
      typedef MP::Matrix<Number> Matrix;
      
      
      template <typename T> Number log(std::complex<T> op);
      template <typename T> Number exp(std::complex<T> op);
      template <typename T> Number sin(std::complex<T> op);
      
#include "complex_impl.h"
      
    } /* naemspace Complex */
  } /* namespace MP */
} /* namespace TkResolver */


#endif /* __TK_MP_COMPLEX_H__ */
