/* -*- c++ -*-
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so by
 * e-mail - mail your message to Viktor T. Toth at <vttoth@vttoth.com>.
 *
 * Updated 2007 by Robert Ham for tkresolver
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh> for tkresolver
 * Copyright (C) Viktor T. Toth at <vttoth@vttoth.com>
 *
 */

#ifndef __TK_MP_LANCZOS_H__
#define __TK_MP_LANCZOS_H__

#include <gmpxx.h>

#include "complex.h"
#include "integer.h"
#include "matrix.h"

namespace TkResolver {
  namespace MP {
    namespace Lanczos {
      
      using namespace TkResolver::MP;
      
      class Context
      {
      private:
	Integer::Number _n;
	Real::Number    _g;
	Real::Matrix      _P;
	unsigned long   _prec;
	
      public:
        Context(const Integer::Number &n_terms = 10, const Real::Number &g = 7.0, unsigned long precision = 1024);
        const Integer::Number &get_n() const;
	const Real::Number &get_g() const;
	const Real::Matrix &get_P() const;
	const unsigned long &get_prec() const;
      };
      
      Real::Number   lngamma(const Real::Number &z, const Context &context = Context());
      Real::Number     gamma(const Real::Number &z, const Context &context = Context());
      
      Complex::Number lngamma(const Complex::Number &z, const Context &context = Context());
      Complex::Number   gamma(const Complex::Number &z, const Context &context = Context());
      
    } /* namespace Lanczos */
  } /* namespace MP */
} /* namespace TkResolver */



#endif /* __TK_MP_LANCZOS_H__ */
